@@ -1,6 +1,6 @@
 ##
 #
-#    Copyright 2004, AllAfrica Global Media
+#    Copyright 2004-2006, AllAfrica Global Media
 #
 #    This file is part of XML::Comma
 #
@@ -66,9 +66,10 @@ sub finish_initial_read {
   # only add_def for top-level and element blocks, and only set and
   # trim the content for the rest (a little cheating is okay in a
   # bootstrap module, right?)
-  if ( $self->tag() eq 'DocumentDefinition' or
-       $self->tag() eq 'element' or
-       $self->tag() eq 'nested_element' ) {
+  my $tag = $self->tag();
+  if ( $tag eq 'DocumentDefinition' or
+       $tag eq 'element' or
+       $tag eq 'nested_element' ) {
     $self->_Def_init_name_up_path ( $in_progress_parser );
     XML::Comma::DefManager->add_def ( $self );
     $self->_config_dispatcher();
@@ -146,6 +147,23 @@ sub bootstrap_block {
   <element><name>include_for_hash</name></element>
   <element><name>plural</name></element>
   <element><name>required</name></element>
+  <nested_element>
+    <name>properties</name>
+    <element><name>ignore_for_hash</name></element>
+    <element><name>include_for_hash</name></element>
+    <element><name>plural</name></element>
+    <element><name>required</name></element>
+    <element><name>nested</name></element>
+    <element><name>blob</name></element>
+    <element><name>enum</name></element>
+    <element><name>boolean</name></element>
+    <element><name>range</name></element>
+    <element><name>timestamp</name></element>
+    <element><name>timestamp_created</name></element>
+    <element><name>timestamp_last_modified</name></element>
+    <element><name>doc_key</name></element>
+    <element><name>single_line</name></element>
+  </nested_element>
   <nested_element>
     <name>class</name>
     <element><name>module</name></element>
@@ -315,10 +333,8 @@ sub bootstrap_block {
   <nested_element>
     <name>index</name>
     <element><name>name</name></element>
-    <!-- 'store' will default to self->element('name')->get() -
-         (note, this is handled by the Store->store() method in
-          the internals ) -->
     <element><name>store</name></element>
+    <element><name>index_from_store</name></element>
     <!-- doc_id_sql_type SHOULD NOT BE CHANGED without completely
          dropping and recreating a given index's database (or otherwise
          altering the database structure outside of Comma). ** there is
@@ -344,7 +360,7 @@ sub bootstrap_block {
       <element><name>code</name></element>
       <element>
         <name>type</name>
-        <default>stringified</default>
+        <default>binary table</default>
       </element>
       <element>
         <name>sql_type</name>
@@ -428,7 +444,8 @@ sub bootstrap_block {
       <name>method</name>
       <defname>DocumentDefinition:method</defname>
     </nested_element>
-    <plural>qw( field
+    <plural>qw( index_from_store
+                field
                 collection
                 bcollection
                 sort
