@@ -1,6 +1,6 @@
 ##
 #
-#    Copyright 2001, AllAfrica Global Media
+#    Copyright 2001-2007, AllAfrica Global Media
 #
 #    This file is part of XML::Comma
 #
@@ -25,6 +25,7 @@ package XML::Comma::Log;
 use Fcntl ":flock";
 use strict;
 
+$XML::Comma::Log::warn_only = 0;
 
 sub err {
   my ( $class, $error_name, $arg2, $doc_id, $extra_text ) = @_;
@@ -33,10 +34,10 @@ sub err {
     my $str = $arg2->info_string_full();
     $str .= " - $extra_text"  if  $extra_text;
     my $error = XML::Comma::Err->new 
-      ( err_name => "$error_name/".$arg2->error_name(),
+      ( err_name => "$error_name/".$arg2->{_err_name},
         info_string => $str,
-        file   => $arg2->file(),
-        line   => $arg2->line(),
+        file   => $arg2->{_file},
+        line   => $arg2->{_line},
         doc_id => $doc_id );
     $class->log ( $error->to_string() )  unless  $internal_eval || $caller_eval;
     die $error;
@@ -57,9 +58,10 @@ sub err {
 
 # usage: XML::Comma::Log->warn ( $string );
 sub warn {
-  chomp $_[1];
-  #print STDERR "$_[1]\n";
-  $_[0]->log ( "WARNING -- $_[1]" );
+  my $msg = $_[1];
+  chomp $msg;
+  #print STDERR "$msg\n";
+  $_[0]->log ( "WARNING -- $msg" );
 }
 
 
@@ -67,15 +69,20 @@ sub log {
   my $string = $_[1];
   chomp $string;
   $string =~ s/\n/ /g;
+  if ($XML::Comma::Log::warn_only) { CORE::warn $string; return; }
   {
     my $log;
-    unless ( open( $log, "+>>", XML::Comma->log_file() ) &&
-	     flock( $log, LOCK_EX ) ) {
-      print STDERR "Can't open " . XML::Comma->log_file();
-      print STDERR time() . "$$ $string";
+    unless ( open( $log, "+>>", XML::Comma->log_file() ) ) {
+      print STDERR "Can't open comma logfile:" . XML::Comma->log_file().
+        " ($@ $!), error follows: " . time() . "$$ $string\n";
       return;
-    };
+    }
+    my $flock_warning = "";
+    unless(flock($log, LOCK_EX)) {
+      $flock_warning = "FLOCK NOT AVAILABLE, LOG MAY BE INCOMPLETE OR CORRUPTED";
+    }
     seek($log, 0, 2); #seek to EOF in case someone appended while we waited
+    print $log "$flock_warning\n" if($flock_warning);
     print $log scalar localtime() . ": $$ $string\n";
     # note close() makes an implicit funlock()
     close($log) || print STDERR "$$ can't close log file at ".time.", other procs may block forever \""
@@ -84,6 +91,7 @@ sub log {
   #`echo "${\( time() )} $$ $string" >> ${ \( XML::Comma->log_file() ) }`;
 }
 
+
 ##
 # first, look for the first stackframe caller that is not an
 # XML::Comma package. then, look all the way up the stack to see if
@@ -96,8 +104,8 @@ sub external_caller {
   my ( $package, $filename, $line, $subroutine,
        $hasargs, $wantarray, $evaltext, $is_require ) = caller($i++);
   while ( $package ) {
-    #print "$package -- $filename -- $line -- $subroutine -- $evaltext\n";
-    #print "CONTEXT: $package -- $line\n";
+    # print "$package -- $filename -- $line -- $subroutine -- $evaltext\n";
+    # print "CONTEXT: $package -- $line\n";
     # first external caller?
     if ( (! $caller_file) and ($package !~ /^XML::Comma/) ) {
       $caller_file = $filename;
@@ -170,5 +178,3 @@ sub to_string {
 }
 
 1;
-
-
