@@ -1,6 +1,6 @@
 ##
 #
-#    Copyright 2004, AllAfrica Global Media
+#    Copyright 2004-2005, AllAfrica Global Media
 #
 #    This file is part of XML::Comma
 #
@@ -22,9 +22,15 @@
 
 package XML::Comma::Pkg::DecoratorTest::Talker;
 
+my $what = "";
+
 sub say_hello {
   my $self = shift;
-  return "hello";
+  return $what;
+}
+
+sub init {
+  $what = "hello";
 }
 
 1;
